#include "ScriptLists.h"


// ********************************************************************************************************
// GENERAL LIST FUNCTIONS
// ********************************************************************************************************

/////////////////////////////////////////////////////////////
// newList
/////////////////////////////////////////////////////////////

FX_EXPFUNC(newList_func, void, data)
{
	fxArgSetInt( return_arg, NULL, 0, newList_internal() );

	return FX_FUNC_OK;
}

FX_FUNC( FXint, newList_internal )( void )
{
	masterListType & master = _getMaster();
	
	master.push_back( new userListType );

	return master.size() - 1;
}
/////////////////////////////////////////////////////////////


/////////////////////////////////////////////////////////////
// deleteList
/////////////////////////////////////////////////////////////

FX_EXPFUNC(deleteList_func, getList_arg, data)
{
	if(!data) return FX_FUNC_FAIL;

	fxArgSetInt( return_arg, NULL, 0, deleteList_internal( data->list ) );

	return FX_FUNC_OK;
}
FX_FUNC( FXint, deleteList_internal )( FXint list )
{
	userListType *ulist = NULL;
	
	if( !( ulist = _getList( list )) )
		return FX_FALSE;

	_killList( list );

	return FX_TRUE;
}
/////////////////////////////////////////////////////////////


/////////////////////////////////////////////////////////////
// getListSize
/////////////////////////////////////////////////////////////

FX_EXPFUNC(getListSize_func, getList_arg, data)
{
	if(!data) return FX_FUNC_FAIL;

	fxArgSetInt( return_arg, NULL, 0, getListSize_internal( data->list) );

	return FX_FUNC_OK;
}

FX_FUNC( FXint, getListSize_internal )( FXint list )
{
	userListType *ulist = NULL;
	
	if( !( ulist = _getList( list )) )
		return -1;

	return ulist->elements.size();
}
/////////////////////////////////////////////////////////////


/////////////////////////////////////////////////////////////
// getListError
/////////////////////////////////////////////////////////////

FX_EXPFUNC(getListError_func, getList_arg, data)
{
	if(!data) return FX_FUNC_FAIL;

	fxArgSetInt( return_arg, NULL, 0, getListError_internal( data->list) );

	return FX_FUNC_OK;
}

FX_FUNC( FXint, getListError_internal )( FXint list )
{
	userListType *ulist = NULL;
	
	if( !( ulist = _getList( list )) )
		return -1;
	
	if( ulist->error )
		return FX_TRUE;
	else
		return FX_FALSE;
}
/////////////////////////////////////////////////////////////


/////////////////////////////////////////////////////////////
// getListErrorString
/////////////////////////////////////////////////////////////

FX_EXPFUNC(getListErrorString_func, getList_arg, data)
{
	if(!data) return FX_FUNC_FAIL;

	fxArgSetString( return_arg, NULL, 0, getListErrorString_internal( data->list) );

	return FX_FUNC_OK;
}

FX_FUNC( FXchar *, getListErrorString_internal )( FXint list )
{
	userListType *ulist = NULL;
	
	if( !( ulist = _getList( list )) )
		return NULL;

	return const_cast<FXchar *>(ulist->errStr.c_str());
}
/////////////////////////////////////////////////////////////


/////////////////////////////////////////////////////////////
// clearListError
/////////////////////////////////////////////////////////////

FX_EXPFUNC(clearListError_func, getList_arg, data)
{
	if(!data) return FX_FUNC_FAIL;

	fxArgSetInt( return_arg, NULL, 0, clearListError_internal( data->list) );

	return FX_FUNC_OK;
}

FX_FUNC( FXint, clearListError_internal )( FXint list )
{
	userListType *ulist = NULL;
	
	if( !( ulist = _getList( list )) )
		return FX_FALSE;

	ulist->error = false;
	ulist->errStr.erase();

	return FX_TRUE;
}
/////////////////////////////////////////////////////////////


/////////////////////////////////////////////////////////////
// sortList
/////////////////////////////////////////////////////////////

FX_EXPFUNC(sortList_func, sortList_arg, data)
{
	if(!data) return FX_FUNC_FAIL;

	fxArgSetInt( return_arg, NULL, 0, sortList_internal( data->list, data->numeric ) );

	return FX_FUNC_OK;
}

FX_FUNC( FXint, sortList_internal )( FXint list, FXint numeric )
{
	userListType *ulist = NULL;

	if( !(ulist = _getList( list )) )
		return FX_FALSE;

	comp_userListElement comp( numeric != 0 );

	std::sort( ulist->elements.begin(), ulist->elements.end(), comp );

	return FX_TRUE;
}
/////////////////////////////////////////////////////////////


/////////////////////////////////////////////////////////////
// copyListToString
/////////////////////////////////////////////////////////////

FX_EXPFUNC(copyListToString_func, copyListToString_arg, data)
{
	if(!data) return FX_FUNC_FAIL;

	fxArgSetString( return_arg, NULL, 0, copyListToString_internal( data->list, data->delimeter ) );

	return FX_FUNC_OK;
}

FX_FUNC( FXchar *, copyListToString_internal )( FXint list, FXchar * delimeter )
{
	static std::string ret;

	userListType *ulist = NULL;

	if( !(ulist = _getList( list )) || ulist->elements.empty() )
		return NULL;

	userListType::element_iterator it = ulist->elements.begin();

	ret = (*it++).as_string(); 

	for( ; it != ulist->elements.end(); it++ )
		ret += delimeter + (*it).as_string();

	return const_cast<FXchar *>( ret.c_str() );
}
/////////////////////////////////////////////////////////////

// ********************************************************************************************************
// LEXICAL CONVERSION FUNCTIONS
// ********************************************************************************************************

//		***		//
//		SET		//
//		***		//

/////////////////////////////////////////////////////////////
// setListInt
/////////////////////////////////////////////////////////////

FX_EXPFUNC(setListInt_func, setListInt_arg, data)
{
	if(!data) return FX_FUNC_FAIL;

	fxArgSetInt( return_arg, NULL, 0, setListInt_internal( data->list, data->index, data->value ) );

	return FX_FUNC_OK;
}

FX_FUNC( FXint, setListInt_internal )( FXint list, FXint index, FXint value )
{
	return _setListElement( list, index, value ) ? FX_TRUE : FX_FALSE;
}
/////////////////////////////////////////////////////////////


/////////////////////////////////////////////////////////////
// setListDouble
/////////////////////////////////////////////////////////////

FX_EXPFUNC(setListDouble_func, setListDouble_arg, data)
{
	if(!data) return FX_FUNC_FAIL;

	fxArgSetInt( return_arg, NULL, 0, setListDouble_internal( data->list, data->index, data->value ) );

	return FX_FUNC_OK;
}

FX_FUNC( FXint, setListDouble_internal )( FXint list, FXint index, FXdouble value )
{
	return _setListElement( list, index, value ) ? FX_TRUE : FX_FALSE;
}
/////////////////////////////////////////////////////////////


/////////////////////////////////////////////////////////////
// setListObject
/////////////////////////////////////////////////////////////

FX_EXPFUNC(setListObject_func, setListObject_arg, data)
{
	if(!data) return FX_FUNC_FAIL;

	fxArgSetInt( return_arg, NULL, 0, setListObject_internal( data->list, data->index, data->value ) );

	return FX_FUNC_OK;
}

FX_FUNC( FXint, setListObject_internal )( FXint list, FXint index, FXobject value )
{
	return _setListElement( list, index, value ) ? FX_TRUE : FX_FALSE;
}
/////////////////////////////////////////////////////////////


/////////////////////////////////////////////////////////////
// setListString
/////////////////////////////////////////////////////////////

FX_EXPFUNC(setListString_func, setListString_arg, data)
{
	if(!data) return FX_FUNC_FAIL;

	fxArgSetInt( return_arg, NULL, 0, setListString_internal( data->list, data->index, data->value ) );

	return FX_FUNC_OK;
}

FX_FUNC( FXint, setListString_internal )( FXint list, FXint index, FXchar * value )
{
	return _setListElement( list, index, value ) ? FX_TRUE : FX_FALSE;
}
/////////////////////////////////////////////////////////////

//		***		//
//		GET		//
//		***		//

/////////////////////////////////////////////////////////////
// getListInt
/////////////////////////////////////////////////////////////

FX_EXPFUNC(getListInt_func, setListInt_arg, data)
{
	if(!data) return FX_FUNC_FAIL;

	fxArgSetInt( return_arg, NULL, 0, getListInt_internal( data->list, data->index ) );

	return FX_FUNC_OK;
}

FX_FUNC( FXint, getListInt_internal )( FXint list, FXint index )
{
	try
	{
		return boost::lexical_cast<FXint>( _getElement( list, index ) );
	}
	catch( const boost::bad_lexical_cast &e )
	{
		#ifdef _DEBUG
			fxMessageSend( FX_NULLID,			"--- ScriptLists Error ---", 
						   FX_MESSAGE_ERROR,	const_cast<FXchar *>(e.what()), "\n*** getListInt ***" );
		#endif

		userListType *ulist = _getList( list );

		if( ulist )
		{
			ulist->error = true;
			ulist->errStr = "Unable to convert value from: " + _getElement( list, index ) + ", to: int";
		}

		return -2147483648;
	}
}
/////////////////////////////////////////////////////////////


/////////////////////////////////////////////////////////////
// getListDouble
/////////////////////////////////////////////////////////////

FX_EXPFUNC(getListDouble_func, setListDouble_arg, data)
{
	if(!data) return FX_FUNC_FAIL;

	fxArgSetDouble( return_arg, NULL, 0, getListDouble_internal( data->list, data->index ) );

	return FX_FUNC_OK;
}

FX_FUNC( FXdouble, getListDouble_internal )( FXint list, FXint index )
{
	try
	{
		return boost::lexical_cast<FXdouble>( _getElement( list, index ) );
	}
	catch( const boost::bad_lexical_cast &e )
	{
		#ifdef _DEBUG
			fxMessageSend( FX_NULLID,			"--- ScriptLists Error ---", 
						   FX_MESSAGE_ERROR,	const_cast<FXchar *>(e.what()), "\n*** getListDouble ***" );
		#endif

		userListType *ulist = _getList( list );

		if( ulist )
		{
			ulist->error = true;
			ulist->errStr = "Unable to convert value from: " + _getElement( list, index ) + ", to: double";
		}

		return 1.7e-308;
	}
}
/////////////////////////////////////////////////////////////


/////////////////////////////////////////////////////////////
// getListObject
/////////////////////////////////////////////////////////////

FX_EXPFUNC(getListObject_func, setListObject_arg, data)
{
	if(!data) return FX_FUNC_FAIL;

	fxArgSetObj( return_arg, NULL, 0, getListObject_internal( data->list, data->index ) );

	return FX_FUNC_OK;
}

FX_FUNC( FXobject, getListObject_internal )( FXint list, FXint index )
{
	FXobject obj;

	try
	{
		obj = boost::lexical_cast<FXobject>( _getElement( list, index ) );

		// make sure its a valid object, otherwise throw a bad_lexical_cast
		if( !fxObjectCheckID( obj ) )
			throw boost::bad_lexical_cast();

		return obj;
	}
	catch( const boost::bad_lexical_cast &e )
	{
		#ifdef _DEBUG
			fxMessageSend( FX_NULLID,			"--- ScriptLists Error ---", 
						   FX_MESSAGE_ERROR,	const_cast<FXchar *>(e.what()), "\n*** getListObject ***" );
		#endif

		userListType *ulist = _getList( list );

		if( ulist )
		{
			ulist->error = true;
			ulist->errStr = "Unable to convert value from: " + _getElement( list, index ) + ", to: object";
		}

		return FX_NULLID;
	}
}
/////////////////////////////////////////////////////////////


/////////////////////////////////////////////////////////////
// getListString
/////////////////////////////////////////////////////////////

FX_EXPFUNC(getListString_func, setListString_arg, data)
{
	if(!data) return FX_FUNC_FAIL;

	fxArgSetString( return_arg, NULL, 0, getListString_internal( data->list, data->index ) );

	return FX_FUNC_OK;
}

FX_FUNC( FXchar *, getListString_internal )( FXint list, FXint index )
{
	// no need to check for a bad lexical cast here as its already done in _getElement
	// the return value will be "list_error" if the string cannot be converted
	return const_cast<FXchar *>( _getElement( list, index ).c_str() );
}
/////////////////////////////////////////////////////////////


// ********************************************************************************************************
// INTERNAL HELPER FUNCTIONS
// ********************************************************************************************************

/////////////////////////////////////////////////////////////
// _setListElement
/////////////////////////////////////////////////////////////

template< typename T >
bool	_setListElement( FXint list, FXint index, T value )
{
	if( list < 0 || index < 0 )
		return false;
	
	masterListType &master = _getMaster();


	// if the list trying to be accessed does not exist
	if( master.empty() || list >= master.size() )
	{
		fxMessageSend( FX_NULLID,			"--- ScriptLists Error ---", 
					   FX_MESSAGE_ERROR,	"Tried to access an invalid list", "\n*** _setListElement" );
		return false;
	}

	userListType *ulist = master[list];

	if( !ulist )
	{
		fxMessageSend( FX_NULLID,			"--- ScriptLists Error ---", 
					   FX_MESSAGE_ERROR,	"Tried to access an invalid list", "\n*** _setListElement" );
		return false;
	}

	try
	{
		// if the index being referenced is beyond the bounds of the list
		// extend the list to the index, filling it with the current value
		// otherwise just set the value
		if( ulist->elements.empty() || index >= ulist->elements.size() )
			ulist->elements.resize( index + 1, boost::lexical_cast<std::string>(value) );
		else
			ulist->elements[index].as_string() = boost::lexical_cast< std::string >( value );

	}
	catch( const boost::bad_lexical_cast &e )
	{
		#ifdef _DEBUG
			fxMessageSend( FX_NULLID,			"--- ScriptLists Error ---", 
						   FX_MESSAGE_ERROR,	const_cast<FXchar *>(e.what()), "\n*** _setListElement" );
		#endif

		return false;
	}

	return true;
}
/////////////////////////////////////////////////////////////


/////////////////////////////////////////////////////////////
// _getElement
/////////////////////////////////////////////////////////////

std::string &		_getElement( int list, int index )
{
	static std::string error("list_error");

	if( list < 0 || index < 0 )
		return error;

	userListType *ulist = _getList(list);

	if( !ulist || ulist->elements.empty() || index >= ulist->elements.size() )
	{
		fxMessageSend( FX_NULLID,			"--- ScriptLists Error ---", 
					   FX_MESSAGE_ERROR,	"Tried to access an invalid list element", "\n*** _getElement" );
		return error;
	}

	return ulist->elements[index].as_string();
}
/////////////////////////////////////////////////////////////


/////////////////////////////////////////////////////////////
// _getList
/////////////////////////////////////////////////////////////
userListType *		_getList( int list )
{
	if( list < 0 )
		return NULL;

	masterListType &master = _getMaster();

	// if the list trying to be accessed does not exist
	if( master.empty() || list >= master.size() )
	{
		fxMessageSend( FX_NULLID,			"--- ScriptLists Error ---", 
					   FX_MESSAGE_ERROR,	"Tried to access an invalid list", "\n*** _getList" );
		return NULL;
	}

	userListType *ulist = master[list];

	if( !ulist )
	{
		fxMessageSend( FX_NULLID,			"--- ScriptLists Error ---", 
					   FX_MESSAGE_ERROR,	"Tried to access an invalid list", "\n*** _getList" );
		return NULL;
	}

	return ulist;
}
/////////////////////////////////////////////////////////////


/////////////////////////////////////////////////////////////
// _killList
/////////////////////////////////////////////////////////////
void		_killList( int list )
{
	if( list < 0 )
		return;

	masterListType &master = _getMaster();

	// if the list trying to be accessed does not exist
	if( master.empty() || list >= master.size() )
	{
		fxMessageSend( FX_NULLID,			"--- ScriptLists Error ---", 
					   FX_MESSAGE_ERROR,	"Tried to access an invalid list", "\n*** _killList" );
		return;
	}

	userListType *ulist = master[list];

	if( ulist )
		delete ulist;

	master[list] = NULL;
}
/////////////////////////////////////////////////////////////


/////////////////////////////////////////////////////////////
// _getMaster
/////////////////////////////////////////////////////////////

masterListType &	_getMaster()
{
	static masterListType theList;

	return theList;
}
/////////////////////////////////////////////////////////////

// ********************************************************************************************************
// <algorithm> FUNCTORS
// ********************************************************************************************************

bool	comp_userListElement::operator()( userListType::element_type a, userListType::element_type b ) const
{
	if( !numericCompare )
		return a.as_string().compare( b.as_string() ) < 0;
	else
	{
		try { return boost::lexical_cast<FXdouble>( a.as_string() ) < boost::lexical_cast<FXdouble>( b.as_string() ); }
		catch( const boost::bad_lexical_cast &e )
		{
			#ifdef _DEBUG
				fxMessageSend( FX_NULLID,			"--- ScriptLists Error ---", 
							   FX_MESSAGE_ERROR,	const_cast<FXchar *>(e.what()), "\n*** comp_userListElement::operator() ***" );
			#endif

			return false;
		}// catch
	}
}

