/*=============================================================================
	Copyright (c) 1999-2004 pmG Worldwide, LLC.

	File:	messiah_mesh.h
	Author:	Lyle Milton
	Date:	01/01/03

		last modified: 5/09/04 - LM
			- added fxMeshPointWeight

	-----------------------------------------------------------------
	MESH TRAVERSAL
	-----------------------------------------------------------------
	There may be times when you need to scan through the points in an
	object's mesh.  The API provides a simple and direct method to
	get point data.

	EX1: Point Traversal

		fxMeshNumPoints(target, &num_points, 0);

		for (i=0;i<num_points;i++)
		{
			//get the point,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,
			fxMeshPoint(target, i, pnt, 0);

			//todo: do something with the point data
			-
			-
			-
		}

	All that is required is to get the number of points and use a
	loop to get the mesh points via fxMeshPoint.  
	
	-----------------------------------------------------------------
	IMPORTANT NOTE: you should never use this to diplace and object's
	points outside of a P_POST_POINT_DISPLACE effect processing message.
	Any attempts to do so will ultimately fail or be otherwise 
	unpredictable.  See messiah_effect.h for more information on
	creating displacement effects, then procede to the next section.



	-----------------------------------------------------------------
	MESH/POINT DISPLACEMENT PROCESSING
	-----------------------------------------------------------------
	If you're creating a displacment effect, once you've set up your
	effect module as noted in messiah_effect.h, you will be able to
	perform mesh displacement in your FX_EFFECTSCAN callback.

	To accomodate the varying needs of plugin developers, messiah has
	2 methods for performing mesh displacent; While both methods must
	be performed inside your FX_EFFECTSCAN callback, you can choose
	the method that best fits your application. One way is to directly
	traverse through the target's points.

	EX2: Processing Points via Point Traversal

		FX_EFFECTSCAN(EffectScan, My_EffectData, data)
		{
			-
			-
			-
			fxMeshNumPoints(target, &num_points, 0);

			fxDisplaceBegin(target, 0);

				for (i=0;i<num_points;i++)
				{
					//get the point,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,
					fxMeshPoint(target, i, pnt, 0);

					//todo: compute your effect's displacement
					-
					-
					-
					//set the point,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,
					fxDisplacePoint(target, i, pos[0], pos[1], pos[2]);
				}

			fxDisplaceEnd(target, 0);

			return FX_TRUE;
		}

	
	As demonstrated in EX2, scanning through the points is rather direct,
	since the point traversal is performed as described in EX1.  The
	major difference is that you must put your point loop between the
	fxDisplaceBegin and fxDisplaceEnd calls.  If you do not do this,
	no points will be diplaced when you call fxDisplacePoint in your loop.
	
	Performing the mesh displacement in this way allows for direct control
	over point scanning.  However, if you don't really need to traverse
	the points yourself, the API includes a mechanism that will traverse
	the points for you, and is also very efficient.  You first need to
	define a displacement scanning callback.


	EX3: Defining a Displacement Scanning Callback

		FXintf MeshScan(FX_DisplacePoint *p, My_EffectData *data)

		- OR -

		FX_DISPLACESCAN(MeshScan, My_EffectData, data)

		{
			//todo: compute the point displacement
			-
			-
			-
			FX_V3COPY(p->new_pos, pos); //set the new position

			return FX_TRUE;
		}


	With your FX_DISPLACESCAN callback defined, all that's remaining is
	to call the diplace scanning API function.

	EX4: Processing Points via Diplace Scan

		FX_EFFECTSCAN(EffectScan, My_EffectData, data)
		{
			-
			-
			-
			fxDisplaceScan(target, FX_NULLID, 0, MeshScan, data, 0);

			return FX_TRUE;
		}


	Your mesh/diplace scan callback will be called for every point in the
	target's mesh.  Since your FX_DISPLACESCAN callback will return an
	FX_DisplacePoint struct pointer that contains all the relevan point
	data, there is no need to call any fxMesh API functions.  To set the
	new position of the point, simply set new position in p->new_pos.

	

=============================================================================*/

#ifdef _MESSIAH_API_MESH

#ifndef __MESSIAH_MESH
#define __MESSIAH_MESH


#ifndef __MESSIAH_HIDE_TYPES

	#define FX_DISPLACESCAN(f, dt, ed)	FXintf f(FX_DisplacePoint *p, dt *ed)
	#define FX_DISPLACESCANMW(f, dt, ed)	FXintf f(FX_DisplacePointMW *p, dt *ed)

	#define FX_WEIGHT_SETUP			(1<<0) //Compute map from setup (will only be updated while in Setup)
	#define FX_WEIGHT_ANIMATE		(1<<1) //Compute map from animate (dynamic map) - will be updated as needed in any mode.
	#define FX_WEIGHT_COMBO			(1<<2) //Compute map based on original points using current weight motion regardless of mode

	#define FX_WEIGHT_DONTCREATE	(1<<31) //don't create weight data if it doesn't exist (fxWeightMapGet)

	//defines for fxMeshPoint, fxMeshPointNormal & fxMeshPolyNormal
	#define	FX_MESHDATA_OBJ			(0)		//get data "as loaded" from the file
	#define	FX_MESHDATA_WORLD		(1<<0)	//get data in world position


#endif //__MESSIAH_HIDE_TYPES


#define FX_DISPFLAG_WEIGHT		(1<<0)

//world screen conversion flags
#define WORLD_TO_SCREEN 0
#define SCREEN_TO_WORLD 1
///////////////

typedef struct FX_DisplacePoint_St
{
	const FXint		objID;
	const FXint		ID;
	const FXvecd	file_pos;
	const FXvecd	cur_pos;
	const FXvecd	normal;
	const FXdouble	weight;

	const FXint		flags;

	FXvecd			new_pos; //new point position

}FX_DisplacePoint;


typedef struct FX_DisplacePointMW_St
{
	const FXint		objID;
	const FXint		ID;
	const FXvecd	file_pos;
	const FXvecd	cur_pos;
	const FXvecd	normal;
	const FXdouble	*weight;
	const FXchar	*weight_flags;
	const FXint		num_weights;

	const FXint		flags;

	FXvecd			new_pos; //new point position

}FX_DisplacePointMW;



FX_API(FXint, fxMeshNumPoints)(FXobject objID, FXint *num_points, FXint flags);

FX_API(FXint, fxMeshNumPolys)(FXobject objID, FXint *num_polys, FXint flags);

FX_API(FXint, fxMeshPoly)(FXobject objID, FXint polyID, FXint pnt_array[], FXint array_size, FXint *num_points, FXint *surfID, FXint flags);

FX_API(FXint, fxMeshPolyNormal)(FXobject objID, FXint polyID, FXdouble norm[3], FXint flags);

FX_API(FXint, fxMeshPoint)(FXobject objID, FXint pointID, FXdouble pos[3], FXint flags);

FX_API(FXint, fxMeshPointNormal)(FXobject objID, FXint pointID, FXdouble norm[3], FXint flags);

FX_API(FXint, fxMeshPointPolys)(FXobject objID, FXint pointID, FXint poly_array[], FXint array_size, FXint *num_polys, FXint flags);

FX_API(FXint, fxMeshPointWeight)(FXobject objID, FXint pointID, FXtool weightID, FXint weight_flags, FXfloat *weight_value, FXint flags);

FX_API(FXint, fxDisplaceScan)(FXobject objID, FXtool weightID, FXint weight_flags, FXint (*scan_func)(FX_DisplacePoint*, FXvoid*), FXvoid *data, FXint flags);

FX_API(FXint, fxDisplaceScanMultiWeight)(FXobject objID, FXveci *weight_data, FXint num_weights, FXint (*scan_func)(FX_DisplacePointMW*, FXvoid*), FXvoid *data, FXint flags);

FX_API(FXint, fxDisplaceBegin)(FXobject objID, FXint flags);

FX_API(FXint, fxDisplacePoint)(FXobject objID, FXint pointID, FXdouble x, FXdouble y, FXdouble z);

FX_API(FXint, fxDisplaceEnd)(FXobject objID, FXint flags);

//flags WORLD_TO_SCREEN or SCREEN_TO_WORLD
FX_API(FXint, fxWorldScreenConversion)(FXdouble *input_pos, FXdouble *result_pos,FXint flags);
///////////

//set, check and clear selection flags on an object's points
FX_API(FXint, fxPointSelect_Set)(FXobject objID, FXint pntID);
FX_API(FXint, fxPointSelect_Check)(FXobject objID, FXint pntID);
FX_API(FXint, fxPointSelect_Clear)(FXobject objID, FXint pntID);
FX_API(FXint, fxPointSelect_ClearAll)(FXobject objID);

///////////////////////////////////

#define _MESSIAH_PTR_MESH	messiah_apicomponent = FX_APIFUNCS_MESH;\
							FX_PTR(fxMeshNumPoints);\
							FX_PTR(fxMeshNumPolys);\
							FX_PTR(fxMeshPoly);\
							FX_PTR(fxMeshPolyNormal);\
							FX_PTR(fxMeshPoint);\
							FX_PTR(fxMeshPointNormal);\
							FX_PTR(fxMeshPointPolys);\
							FX_PTR(fxMeshPointWeight);\
							FX_PTR(fxDisplaceScan);\
							FX_PTR(fxDisplaceScanMultiWeight);\
							FX_PTR(fxDisplaceBegin);\
							FX_PTR(fxDisplacePoint);\
							FX_PTR(fxDisplaceEnd);\
							FX_PTR(fxWorldScreenConversion);\
							FX_PTR(fxPointSelect_Set);\
							FX_PTR(fxPointSelect_Check);\
							FX_PTR(fxPointSelect_Clear);\
							FX_PTR(fxPointSelect_ClearAll)


#endif //__MESSIAH_MESH

#endif //_MESSIAH_API_MESH

