/*###################################################################################
=====================================================================================
	messiahrenderHOST.h

	Authored by Lyle Milton - 11/20/02
	Copyright (c) 1998-2002 pmG Worldwide, LLC. All Rights Reserved.

	This is the header for the messiah:render Host API.  V 1.0


=====================================================================================
###################################################################################*/

#ifndef MESSIAH_RENDER_HOST_DLL
#define MESSIAH_RENDER_HOST_DLL

#define MESSIAH_RHAPI_VERSION		(0x00100002)

//Set any combination of these flags to be notified during rendering
//EX:
//		render_host_info.flags = RHI_CALLBACK_START|RHI_CALLBACK_END;
//
//When your callback is called, the op arg will be the current notification.
//EX:
//		switch (op)
//		{
//			case RHI_CALLBACK_START:
//
#define RHI_CALLBACK_START			(1<<0) //renderer has started
#define RHI_CALLBACK_END			(1<<1) //renderer is done
#define RHI_CALLBACK_FRAME_START	(1<<2) //new frame has been started
#define RHI_CALLBACK_FRAME_END		(1<<3) //current frame is done
#define RHI_CALLBACK_SCANLINE		(1<<4) //NOT ACTIVE
#define RHI_CALLBACK_MESSAGES		(1<<5) //NOT ACTIVE
#define RHI_CALLBACK_FRAME_SET		(1<<6) //allow host to set the frame that will be rendered next - set rhi->current_frame to the frame you want to render, or -1 to stop rendering
											//remember to set this flag to tell the system you want to set frames yourself

#define RHI_BUFFERTYPE_COLOR		"Color"
#define RHI_BUFFERTYPE_ALPHA		"Alpha"
#define RHI_BUFFERTYPE_DEPTH		"Depth"


//This stuct type is used to "register" your program with the renderer.
typedef struct FX_RenderHostInfo_St
{
	int			flags;
	int			(*hostCallback)(struct FX_RenderHostInfo_St*, int op, int notifiy_flags);

	//NOTE=======================================================================
	//You, the developer, are responsible for allocating EITHER of these buffers.
	//You must use fxRenderImageSizeGet to get the image size to allocate your buffers.
	//	frame_buffer size = width*height*3
	//	scanline size = width*3
	//DON'T ALLOCATE THESE YET, THEY'RE NOT CURRENTLY BEING ACCESSED
	char		*frame_buffer; //NOT CURRENTLY USED
	char		*scanline; //NOT CURRENTLY USED


	char		message[2048]; //if you want messages, this buffer will be filled in when a message is sent
	int			messageID;

	int			current_frame;
	int			current_scanline;

	void		*data; //do whatcha like

}FX_RenderHostInfo;


//Render Options----------------------------------------------------------------------
//Use these flags with renderOptionsSet/Clear.  These flags will also be returned when you
//call renderOptionsGet
#define RHOP_SAVE_FILES				(1<<0)
#define RHOP_LIMITED_REGION			(1<<1)

//Use with renderRangeSet
#define RHR_SINGLE_FRAME			(1<<2) //render a single frame only

//=====================================================================================
//Begin HOST Specific  ================================================================
//=====================================================================================

#ifdef MESSIAH_HOST_MODULE

#define MESSIAH_MODULE_NAME		"messiahHOST.dll"

/* These extentions are no longer valid.  The format list is much larger and may be
	expanded.  Because of this, you must first request the number of formats, then 
	get the format names (not just extensions).  You can use those names to display 
	in your host program.

#define RHFORMAT_TGA	".tga"
#define RHFORMAT_BMP	".bmp"
#define RHFORMAT_JPG	".jpg"
#define RHFORMAT_AVI	".avi" //NOT CURRENTLY SUPPORTED
*/

#define RHFTYPE_CURRENT		0
#define RHFTYPE_FORMAT		0
#define RHFTYPE_EXTENSION	1
#define RHFTYPE_ID			2


#ifdef HOST_MODULE
	#define HOSTVAR(t,v)	t *v
	#define HOSTFUNC(t, f) t (*f)
#else
	#define HOSTVAR(t,v)	extern t *v
	#define HOSTFUNC(t, f) extern t (*f)
#endif


//Important for using C++ functions & variables in messiah,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,
//
//Usage:
// _EXPORTVAR(int) variable;
// _EXPORTVAR(double) junk = 0.0;
// _EXPORTVAR(void*) hostFunc(void *data, int val){ }
#if defined(__cplusplus)

	#define __EXPORTVAR(t)	extern "C" t

#else

	#define __EXPORTVAR(t)	t

#endif



//These are the functions that the host calls to establish and manage the messiah connection
#if defined(__cplusplus)
extern "C"
{
#endif

/*===============================================================================
MESSIAH GLOBALS
	This function is used to retrieve the following globals from messiah.  It
	should normally be called from within a host-supplied "hostGlobalsGetAll"
	(or some suched named) function.

	EX:
		void hostGlobalsGetAll(void)
		{
			messiahStart = messiahGlobal("messiahStart");
			messiahInterface = messiahGlobal("messiahInterface");
			.
			.
			.
		}
	
	Your hostGlobalsGetAll() function should be called right after loading the 
	messiahHOST.dll.
===============================================================================*/

	HOSTFUNC(void*, messiahGlobal)(char global_name[]);


/*===============================================================================
MESSIAH SYSTEM MANAGEMENT
	These are the necessary pointers to activate, manage, and shut down messiah.
	Without these functions there can be no messiah-to-HOST connection.
===============================================================================*/

	HOSTFUNC(int, messiahStart)(char host_name[], char module_name[], void *(*globalfunc)(char *), int api_version);
	HOSTFUNC(void, messiahShutdown)(void);

/*===============================================================================
RENDER SYSTEM
	This system allows messiah:render to be controlled via external host module.
	Since messiah scene files contain all the render settings, only
		- renderSceneLoad
		- renderBegin
	are required, and in that order.  Use all others as desired.
===============================================================================*/

	HOSTFUNC(int, renderBegin)(void *rhi, int pre_roll_frame, int flags);
	HOSTFUNC(int, renderStop)(int flags);
	HOSTFUNC(int, renderInProgress)(void);
	HOSTFUNC(int, renderNumThreadsSet)(int num_threads, int flags);
	HOSTFUNC(int, renderNumThreadsGet)(int *num_threads, int flags);
	HOSTFUNC(int, renderSceneLoad)(char *file_path);
	HOSTFUNC(int, renderSceneClear)(void);
	HOSTFUNC(int, renderRangeSet)(int start, int end, int flags);
	HOSTFUNC(int, renderRangeGet)(int *start, int *end, int flags);
	HOSTFUNC(int, renderStepSet)(int frame_step, int flags);
	HOSTFUNC(int, renderStepGet)(int *frame_step, int flags);
	HOSTFUNC(int, renderBufferNumTypes)(int flags);
	HOSTFUNC(int, renderBufferTypeGet)(int ID, char *buffer_type, int buffer_length, int flags);
	HOSTFUNC(int, renderBufferOptionsSet)(char *buffer_type, int option_flags);
	HOSTFUNC(int, renderBufferOptionsClear)(char *buffer_type, int option_flags);
	HOSTFUNC(int, renderBufferOptionsGet)(char *buffer_type, int *option_flags);
	HOSTFUNC(int, renderImageNumFormats)(int flags);
	HOSTFUNC(int, renderImageFileTypeSet)(char *buffer_type, char *extension, int ID, int flags);
	HOSTFUNC(int, renderImageFileTypeGet)(char *buffer_type, char *extension, int buffer_length, int ID, int flags);
	HOSTFUNC(int, renderImageSizeSet)(int width, int height);
	HOSTFUNC(int, renderImagePathSet)(char *buffer_type, char *file_path, int flags);
	HOSTFUNC(int, renderImagePathGet)(char *buffer_type, char *file_path, int buffer_length, int flags);
	HOSTFUNC(int, renderImageNameSet)(char *buffer_type, char *image_name, int flags);
	HOSTFUNC(int, renderImageNameGet)(char *buffer_type, char *image_name, int buffer_length, int flags);
	HOSTFUNC(int, renderImageSizeGet)(int *width, int *height);
	HOSTFUNC(int, renderImageRegionSet)(float bottom, float left, float top, float right, int flags);
	HOSTFUNC(int, renderImageRegionGet)(float *bottom, float *left, float *top, float *right, int flags);
	HOSTFUNC(int, renderOptionsSet)(int option_flags);
	HOSTFUNC(int, renderOptionsClear)(int option_flags);
	HOSTFUNC(int, renderOptionsGet)(int *option_flags);
	HOSTFUNC(int, renderSequenceDigitsSet)(int num_digits);
	HOSTFUNC(int, renderSequenceDigitsGet)(int *num_digits);
	HOSTFUNC(int, renderAASetNone)(void);
	HOSTFUNC(int, renderAASetNormal)(void);
	HOSTFUNC(int, renderAASetEnhanced)(void);
	HOSTFUNC(int, renderAASetSampleLevel)(int level);


#if defined(__cplusplus)
}
#endif

#else

	#ifndef _HIDE_RENDERHOST_FUNCS
		int	(*hostSetContentDirectory)(char *content_buffer, int buff_length, int flags);
	#endif


#endif //MESSIAH_HOST_MODULE
#endif //MESSIAH_RENDER_HOST_DLL


