/*=============================================================================
  Copyright (c) Pekka Kytl

  File: useful_Brush.h

  Author: Pekka Kytl
  Date: 9:00 2008/12/15

  Description:
	Brush system for messiah

=============================================================================*/

//modes
#define MODE_POINT			0 //for weightmap painting use this
#define MODE_VECTOR_STATIC	1 //for basic vertex dragging use this
#define MODE_VECTOR_DYNAMIC 2 //dynamic weight calculation for smudge like effect
#define MODE_SELECT_VERTEX	3
#define MODE_SELECT_EDGE	4 
#define MODE_SELECT_LOOP	5 //edge loop
#define MODE_SELECT_POLYGON	6
#define MODE_SELECT_PAINT	7
#define MODE_SELECT_BOX		8

typedef struct FXbrush
{
	//input/settings
	FXint mode;

	FXint pickThrough;
	FXint weightConnected;

	FXdouble radius;
	FXdouble falloff;

	//targets
	FXint targetCount;
	FXobject *target;		//target id's

	//selection[target][point]
	FXint **selection;
	FXdouble **weight;

	//mouse data
	FXint mouseX;
	FXint mouseY;
	FXint mouseB; //button, FX_HANMBUTTON_LEFT or FX_HANMBUTTON_MIDDLE or FX_HANMBUTTON_RIGHT

	FXobject hitTarget;
	FXint	 hitPolygon;
	FXint	 hitTriangle[3];
	FXint	 hitEdge[2];
	FXint	 hitVertex;
	FXvecd	 hitBarycentric;
	FXvecd	 hitIntersection; //== t*raydirection+raystart
	FXdouble hitLength; // == t

	FXvecd position;
	FXvecd vector;

	FXint **dragboxstate;	//this is used only when selecting by dragging a box...
							//a bit ugly yes, but it gives us nice feedback

	struct Mesh **trimesh;		//store triangulated targets...

	//procedural used to visualize brush
	FXtool manipulator;

	//undo data
	FXint undoid;
	FXvoid (*undostackput)(struct FXbrush *brush, FXvoid *undodata, FXint undo);
	FXvoid (*undostackpop)(struct FXbrush *brush, FXvoid *undodata, FXint undo); 
	FXvoid *undodata;
	FXint undosize;

	//data for internal use only
	FXchar *dataKey;
	FXaction actionId;
	FXhandle handleId;
	FXarmature armatureId;

	FXint viewtype;

}FXbrush;


//data management
FX_API(FXint, fxCreateBrush)(FXvoid *key, FXint keyID);
FX_API(FXbrush *, fxFetchBrush)(FXint brushID, FXvoid *key, FXint keyID);
FX_API(FXint, fxDeleteBrush)(FXint brushID, FXvoid *key, FXint keyID);

//target management
FX_API(FXvoid, fxBrushAddTarget)(FXbrush *brush, FXobject target);
FX_API(FXvoid, fxBrushRemoveTarget)(FXbrush *brush, FXobject target);
FX_API(FXint, fxBrushFindTargetOrder)(FXbrush *brush, FXobject target);

//selection
FX_API(FXvoid, fxBrushSelectVertex)(FXbrush *brush, FXobject target, FXint vertex);
FX_API(FXvoid, fxBrushDeselectVertex)(FXbrush *brush, FXobject target, FXint vertex);
FX_API(FXvoid, fxBrushSelectEdge)(FXbrush *brush, FXobject target, FXint edge[2]);
FX_API(FXvoid, fxBrushDeselectEdge)(FXbrush *brush, FXobject target, FXint edge[2]);
FX_API(FXvoid, fxBrushSelectPolygon)(FXbrush *brush, FXobject target, FXint polygon);
FX_API(FXvoid, fxBrushDeselectPolygon)(FXbrush *brush, FXobject target, FXint polygon);
FX_API(FXvoid, fxBrushSelectTarget)(FXbrush *brush, FXobject target);
FX_API(FXvoid, fxBrushDeselectTarget)(FXbrush *brush, FXobject target);
FX_API(FXvoid, fxBrushSelectAll)(FXbrush *brush);
FX_API(FXvoid, fxBrushDeselectAll)(FXbrush *brush);
FX_API(FXvoid, fxBrushInvertSelection)(FXbrush *brush);
FX_API(FXvoid, fxBrushSetSelection)(FXbrush *brush, FXobject target, FXint *selection);
FX_API(FXint *, fxBrushGetSelection)(FXbrush *brush, FXobject target);
FX_API(FXvoid, fxBrushUpdateSelection)(FXbrush *brush);

//raycast, drag-a-box-selection
FX_API(FXint, fxBrushRayCast)(FXbrush *brush);
FX_API(FXvoid, fxBrushDragBoxSelection)(FXbrush *brush, FXint cull);
FX_API(FXvoid, fxBrushCalculateWeights)(FXbrush *brush);

//check undo
FX_API(FXvoid, fxBrushCheckUndo)(FXbrush *brush);


#define _USEFUL_BRUSH	fxCreateBrush = fxGetAPIFunc("useful_CreateBrush", FX_APIFUNCS_CUSTOM);\
						fxFetchBrush = fxGetAPIFunc("useful_FetchBrush", FX_APIFUNCS_CUSTOM);\
						fxDeleteBrush = fxGetAPIFunc("useful_DeleteBrush", FX_APIFUNCS_CUSTOM);\
						fxBrushAddTarget = fxGetAPIFunc("useful_BrushAddTarget", FX_APIFUNCS_CUSTOM);\
						fxBrushRemoveTarget = fxGetAPIFunc("useful_BrushRemoveTarget", FX_APIFUNCS_CUSTOM);\
						fxBrushFindTargetOrder = fxGetAPIFunc("useful_BrushFindTargetOrder", FX_APIFUNCS_CUSTOM);\
						fxBrushSelectVertex = fxGetAPIFunc("useful_BrushSelectVertex", FX_APIFUNCS_CUSTOM);\
						fxBrushDeselectVertex = fxGetAPIFunc("useful_BrushDeselectVertex", FX_APIFUNCS_CUSTOM);\
						fxBrushSelectEdge = fxGetAPIFunc("useful_BrushSelectEdge", FX_APIFUNCS_CUSTOM);\
						fxBrushDeselectEdge = fxGetAPIFunc("useful_BrushDeselectEdge", FX_APIFUNCS_CUSTOM);\
						fxBrushSelectPolygon = fxGetAPIFunc("useful_BrushSelectPolygon", FX_APIFUNCS_CUSTOM);\
						fxBrushDeselectPolygon = fxGetAPIFunc("useful_BrushDeselectPolygon", FX_APIFUNCS_CUSTOM);\
						fxBrushSelectTarget = fxGetAPIFunc("useful_BrushSelectTarget", FX_APIFUNCS_CUSTOM);\
						fxBrushDeselectTarget = fxGetAPIFunc("useful_BrushDeselectTarget", FX_APIFUNCS_CUSTOM);\
						fxBrushSelectAll = fxGetAPIFunc("useful_BrushSelectAll", FX_APIFUNCS_CUSTOM);\
						fxBrushDeselectAll = fxGetAPIFunc("useful_BrushDeselectAll", FX_APIFUNCS_CUSTOM);\
						fxBrushInvertSelection = fxGetAPIFunc("useful_BrushInverSelection", FX_APIFUNCS_CUSTOM);\
						fxBrushSetSelection = fxGetAPIFunc("useful_BrushSetSelection", FX_APIFUNCS_CUSTOM);\
						fxBrushGetSelection = fxGetAPIFunc("useful_BrushGetSelection", FX_APIFUNCS_CUSTOM);\
						fxBrushUpdateSelection = fxGetAPIFunc("useful_BrushUpdateSelection", FX_APIFUNCS_CUSTOM);\
						fxBrushRayCast = fxGetAPIFunc("useful_BrushRayCast", FX_APIFUNCS_CUSTOM);\
						fxBrushDragBoxSelection = fxGetAPIFunc("useful_BrushDragBoxSelection", FX_APIFUNCS_CUSTOM);\
						fxBrushCalculateWeights = fxGetAPIFunc("useful_BrushCalculateWeights", FX_APIFUNCS_CUSTOM);\
						fxBrushCheckUndo = fxGetAPIFunc("useful_BrushCheckUndo", FX_APIFUNCS_CUSTOM);


